@@ -9,10 +9,11 @@
 """renderer objects"""
 
 __all__ = ['Renderer', 'RadianceRenderer', 'Rtrace', 'Rcontrib',
-           'ImageRenderer']
+           'ImageRenderer', 'SpRenderer']
 
 from raytraverse.renderer.renderer import Renderer
 from raytraverse.renderer.radiancerenderer import RadianceRenderer
 from raytraverse.renderer.rtrace import Rtrace
 from raytraverse.renderer.rcontrib import Rcontrib
 from raytraverse.renderer.imagerenderer import ImageRenderer
+from raytraverse.renderer.sprenderer import SpRenderer
