@@ -62,7 +62,7 @@ class Rtrace(RadianceRenderer):
     # defaultargs = "-av 0 0 0 -aa 0 -ab 7 -ad 128 -as 0 -c 10 -as 0 -lw 1e-5"
     defaultargs = (f"-u+ -ab 16 -av 0 0 0 -aa 0 -as 0 -dc 1 -dt 0 -lr -14 -ad "
                    f"1000 -lw 0.00004 -st 0 -ss 16 -w-")
-    directargs = "-w- -av 0 0 0 -ab 0 -lr 0 -n 1"
+    directargs = "-w- -av 0 0 0 -ab 0 -lr 1 -n 1"
     usedirect = False
     ospec = "Z"
     ocnt = 1
@@ -96,7 +96,7 @@ class Rtrace(RadianceRenderer):
 
         """
         super().set_args(args, nproc)
-        ospec = re.findall("-o\w+", cls.args)
+        ospec = re.findall(r"-o\w+", cls.args)
         if len(ospec) > 0:
             cls.update_ospec(ospec[-1][2:])
 
@@ -183,3 +183,8 @@ class Rtrace(RadianceRenderer):
                       f"no new ambfile was specified, stripping {hasamb[-1]} "
                       "from args", file=sys.stderr)
             cls.set_args(args)
+
+    @classmethod
+    def get_sources(cls):
+        """x,y,z,radius,area (x,y,z iis direction for distant sources)"""
+        return cls.instance.get_sources().reshape(-1, 5)
