@@ -116,8 +116,8 @@ class SamplerSuns(BaseSampler):
         vfile = (f"{self.scene.outdir}/{areamapper.name}/{skymapper.name}_"
                  f"{self.stype}.tsv")
         try:
-            suns = np.loadtxt(vfile)
-        except OSError:
+            suns = io.load_txt(vfile)
+        except FileNotFoundError:
             suns = None
         except ValueError:
             suns = None
@@ -155,7 +155,11 @@ class SamplerSuns(BaseSampler):
         # reset/initialize runtime properties
         self._skymapper = skymapper
         self._areamapper = areamapper
-        self._specguide = specguide
+        reflf = f"{self.scene.outdir}/{areamapper.name}/reflection_normals.txt"
+        if specguide is True:
+            self._specguide = reflf
+        else:
+            self._specguide = specguide
         self._areaweights = None
         levels = self.sampling_scheme(skymapper)
         super().run(skymapper, areamapper.name, levels, **kwargs)
