@@ -1,5 +1,5 @@
 #ifndef lint
-static const char	RCSid[] = "$Id: ambcomp.c,v 2.86 2021/02/17 01:29:22 greg Exp $";
+static const char	RCSid[] = "$Id: ambcomp.c,v 2.88 2021/12/15 01:38:50 greg Exp $";
 #endif
 /*
  * Routines to compute "ambient" values using Monte Carlo
@@ -25,8 +25,6 @@ static const char	RCSid[] = "$Id: ambcomp.c,v 2.86 2021/02/17 01:29:22 greg Exp
 #define MINADIV		7	/* minimum # divisions in each dimension */
 #endif
 
-extern void		SDsquare2disk(double ds[2], double seedx, double seedy);
-
 typedef struct {
 	COLOR	v;		/* hemisphere sample value */
 	float	d;		/* reciprocal distance */
@@ -100,7 +98,8 @@ ambsample(				/* initial ambient division sample */
 	AMBSAMP	*ap = &ambsam(hp,i,j);
 	RAY	ar;
 	int	hlist[3], ii;
-	double	spt[2], zd;
+	RREAL	spt[2];
+	double	zd;
 					/* generate hemispherical sample */
 					/* ambient coefficient for weight */
 	if (ambacc > FTINY)
@@ -118,7 +117,7 @@ ambsample(				/* initial ambient division sample */
 	hlist[2] = i;
 	multisamp(spt, 2, urand(ilhash(hlist,3)+n));
 resample:
-	SDsquare2disk(spt, (j+spt[1])/hp->ns, (i+spt[0])/hp->ns);
+	square2disk(spt, (j+spt[1])/hp->ns, (i+spt[0])/hp->ns);
 	zd = sqrt(1. - spt[0]*spt[0] - spt[1]*spt[1]);
 	for (ii = 3; ii--; )
 		ar.rdir[ii] =	spt[0]*hp->ux[ii] +
@@ -765,7 +764,7 @@ doambient(				/* compute ambient component */
 				ra[0] = maxarad;
 		}
 					/* flag encroached directions */
-		if (crlp != NULL)
+		if (crlp != NULL)	/* XXX doesn't update with changes to ambacc */
 			*crlp = ambcorral(hp, uv, ra[0]*ambacc, ra[1]*ambacc);
 		if (pg != NULL) {	/* cap gradient if necessary */
 			d = pg[0]*pg[0]*ra[0]*ra[0] + pg[1]*pg[1]*ra[1]*ra[1];
@@ -779,4 +778,3 @@ doambient(				/* compute ambient component */
 	free(hp);			/* clean up and return */
 	return(1);
 }
-
