@@ -50,8 +50,7 @@ def test_skysample(tmpdir):
 
     scene = Scene('skysample', "box.rad", frozen=False)
     rcontrib = Rcontrib('-ab 1 -ad 10000 -c 1 -lw 1e-5', scene.scene)
-    print(rcontrib)
-    sampler = SkySamplerPt(scene, rcontrib, fdres=7)
+    sampler = SkySamplerPt(scene, rcontrib, nlev=3)
     vm = ViewMapper((0, 1, 0), viewangle=180)
     lf = sampler.run((1.5, 1.5, 1.5), 0)
     illum, illumm = img_illum(lf, vm, 176)
@@ -72,7 +71,8 @@ def test_skysample(tmpdir):
 def test_sunsample(tmpdir):
 
     def img_illum(lf0, vm0):
-        outf = lf0.direct_view(res=512, showsample=False, scalefactor=285.32)
+        outf = lf0.direct_view(res=512, showsample=False, scalefactor=285.32,
+                               vm=vm0)
         hdr = io.hdr2array(outf)
         vecs = vm0.pixelrays(512)
         pxs = vm0.pixels(512).reshape(-1, 2)
